;,-----------------------------------------------------------------,
;| Popsproc.asm - procedure koje se pozivaju iz glavnog programa   |
;\_________________________________________________________________'
 extrn 	string:byte,stringduz:byte	; javne promenljive iz glavnog programa
 extrn 	casesens:byte			; sa deklaracijom tipa
 KOD 	segment para public 'code'
  assume CS:KOD, DS:KOD, ES:KOD
  public SEEK,PisiBP,GrOtv,GrCit	; javne procedure koje se koriste u gl.programu
  
  quant		equ	8192		; broj bajtova koji se odjednom cita iz fajla (8K)

  bafer		db	quant+254 dup (?)	; bafer (quant+254) bajtova
  even					; sledeca rec na parnu adresu
  baferduz	dw	?		; trenutni broj bajtova u baferu	
  broj		db	5 dup ("0"),"$" ; broj ponavljanja u ASCII formatu
  
  greskaopen	db	13,10,7,"!!! GRESKA pri otvaranju datoteke !!!$"
  greskaread	db	13,10,7,"!!! GRESKA pri citanju iz datoteke !!!$"
 
;,-----------------------------------------------------------------,
;| SeeK                                                            |
;| Procedura za prebrojavanje pojavljivanja stringa u fajlu        |
;|  podrazumeva se sledeci prenos parametara 			   |
;|   * ulaz:							   |
;|     - string se nalazi u KOD segmentu na adresi string	   |
;|     - duzina stringa je na adresi stringduz			   |
;|     - file handle datoteke za pretrazivanje je u BX		   |
;|   * izlaz:							   |
;|     - povratna vrednost (broj ponavljanja) se vraca preko BP	   |
;|   * Registri se koriste u sledece svrhe:			   |
;|     - AX - duzina bafera i za poziv dos funkcija INT 21h 	   |
;|     - BX - file handle					   |
;|     - CX - brojac u string instrukcijama i pri citanju	   |
;|     - DX - duzina stringa					   |
;|     - SI - pokazivac na bafer				   |
;|     - DI - pokazivac na string (i po potrebi na bafer)	   |
;|     - BP - Brojac Ponavljanja stringa			   |
;|   * menjaju se svi navedeni registri	i flagovi		   |
;|  VAZNO: 							   |
;|   - maximalni korektno registrovani broj ponavljanja je 65535   |
;\_________________________________________________________________'
  SEEK 	PROC near
  	cmp casesens,1		; da li je case sensitive pretrazivanje
  	je  initBP		; ako jeste ne menja se string
  	call smanstr		; ako nije smanjuju mu se sva velika slova
  	
 initBP:xor BP,BP		; pocetna vrednost brojaca ponavljanja stringa
 	
 	mov CX,quant		; pocetno punjenje bafera
 	mov DX,offset bafer
 	mov AH,3Fh
 	int 21h
 	jnc initRx		; nije bilo greske
 	call grcit		; greska pri citanju iz fajla
 	
 initRx:xor DH,DH
 	xor CH,CH
 	mov DL,stringduz	; DX <- duzina stringa
 	cmp AX,DX		; br.procitanih bajtova - duzina stringa
 	jb  KRAJP		; ako je manje od nule izlaz
 	mov baferduz,AX		
 	mov SI,offset bafer
 petlja:mov DI,offset string
 	mov CL,stringduz	; CX <- duzina stringa
 	cmp casesens,1		; da li je case sensitive pretrazivanje
 	je  cssc		; jeste
 	call cisc		; Case Insensitive String Compare
 	jmp dalje
 	
  cssc:	repe cmpsb		; Case Sensitive String Compare
  dalje:jne razl
 	
 	inc BP			; konstatovano je poklapanje
 	mov AX,baferduz
 	sub AX,DX		; duzina bafera -= duzina stringa
 	mov baferduz,AX
  dodaj:cmp AX,DX		; duzina bafera - duzina stringa
 	jae petlja		; ima dovoljno bajtova u baferu
 	mov CX,AX		; nema dovoljno bajtova u baferu
 	mov DI,offset bafer	; DI ukazuje sada na pocetak bafera
 	rep movsb		; premesta ostatak bafera na pocetak
 	
 	mov CX,quant		; novi quant
 	mov DX,DI		; pozicija u baferu gde treba dodati
 	mov AH,3Fh		; CITANJE iz fajla
 	int 21h
 	jnc clrRx
 	call grcit		; greska pri citanju iz fajla	
 	jmp KRAJP		; izaziva nasilan izlaz iz procedure
 clrRx:	xor DH,DH
 	xor CH,CH
 		
 	add AX,baferduz		; nova duzina bafera=stara+broj procitanih B
 	mov DL,stringduz
 	cmp AX,DX		; duzina bafera - duzina stringa
 	jb  KRAJP		; duzina bafera je opet manja od stringa
 	mov baferduz,AX
 	mov SI,offset bafer
 	jmp petlja
 	
  razl: 			; nisu se poklopili stringovi
  	sub DX,CX		; broj pocetnih poklopljenih karaktera
  	dec DX
  	sub SI,DX		; pozicija drugog znaka prethodno proveravanog stringa
 				; je pocetak novog stringa za proveru
 	mov AX,baferduz
 	dec AX
 	mov baferduz,AX
 	mov DL,stringduz
 	jmp dodaj		; eventualno dopunjavanje bafera
 	
  KRAJP:ret			; izlaz iz procedure
  SEEK ENDP			; kraj procedure
;,-----------------------------------------------------------------,
;| Procedura za prebacivanje stringa (iz memorije) u mala slova	   |
;|  * ulaz:							   |
;|    - string je u memoriji na labeli string			   |	
;|    - duzina stringa je u memoriji na labeli stringduz	   |
;|  * izlaz:							   |
;|    - string sa malim slovima na adresi string		   |
;|  * korisceni (izmenjeni) registri:				   |	
;|    - SI,DI,CX,AL,flagovi					   |
;\_________________________________________________________________'
smanstr PROC near
	xor CH,CH
	mov CL,stringduz	; CX <- duzina stringa
	mov SI,offset string	; SI <- pocetak stringa
	mov DI,SI		; DI <- pocetak stringa
 sledsl:lodsb			; sledeci znak stringa u AL
 	call smansl		; smanji slovo iz AL ako je veliko
 	stosb			; vrati ga u string
 	loop sledsl		; sledeci znak ako ih jos ima
 	ret			; izlaz iz procedure
smanstr endp			; kraj procedure
;,-----------------------------------------------------------------,
;| Procedura za smanjivanje (eventualnog) velikog slova u AL	   |
;|  * ulaz:							   |
;|    - znak u AL                                       	   |
;|  * izlaz:							   |
;|    - znak u AL                               		   |
;|  * korisceni (izmenjeni) registri:				   |	
;|    - AL,flagovi         					   |
;\_________________________________________________________________'
 smansl	PROC near
	cmp AL,"A"		; AL-"A"
	jb  IZLAZ1		; nije slovo
	cmp AL,"Z"		; AL-"Z"
	ja  IZLAZ1		; nije veliko slovo
	add AL,32		; jeste veliko slovo i smanjuje se
 IZLAZ1:ret	
 smansl	endp
;,-----------------------------------------------------------------,
;| Procedura za Case Insensitive String Compare			   |
;|   radi slicno sto i repe cmpsb osim sto je case INsensitive 	   |
;|  * ulaz:							   |
;|    - CX - duzina stringova					   |
;|    - SI - pocetak izvorisnog stringa				   |
;|    - DI - pocetak odredisnog stringa bez velikih slova	   |
;|  * izlaz:							   |
;|    - CX - broj preostalih (neuporedjivanih) znakova stringova   |
;|    - SI - za jednake stringove: prvi znak posle kraja stringa   |
;|           za razlicite: prvi znak posle prvog razlicitog znaka  |
;|           izvorisnog stringa					   |
;|    - DI - isto kao SI, samo za odredisni string		   |
;|    - ZF - da li su jednaki stringovi				   |
;|  * korisceni (izmenjeni) registri:				   |	
;|    - CX,SI,DI,AL,flagovi					   |
;\_________________________________________________________________'
 cisc	PROC near 	
 CMPSL: jcxz IZLAZ2		; ako je CX=0 izlaz
 	dec  CX			
 	lodsb			; ucitava sledeci znak izvorisnog stringa
 	scasb			; poredi ga sa sledecim iz odredisnog stringa 
 	je   CMPSL		; ako su jednaki ide na sldeci znak
 	call smansl		; ako nisu smanjuje tekuce slovo izvorisnog
 	dec  DI			; vraca pokazivac odredisnog stringa na tekuci znak
 	scasb			; ponovo poredi
 	je   CMPSL		; ako su jednaki ide na sledeci znak
 IZLAZ2:ret
 cisc	endp
;,-----------------------------------------------------------------,
;| Procedura za ispis decimalnee vrednosti registra BP		   |
;|  * ulaz:							   |
;|    - BP - broj za ispis   					   |
;|    - 6 bajtova u memoriji na lebeli broj (00000$)		   |
;|  * izlaz:							   |
;|    - broj predtsavljen kao string u memoriji na labeli broj     |
;|  * korisceni (izmenjeni) registri:				   |	
;|    - DI,AX,BX,CX,DX,flagovi					   |
;\_________________________________________________________________'
 PisiBP PROC near
 	mov DI,offset broj + 4	; poslednja cifra
 	mov AX,BP		; broj za ispis
 	mov BX,10		; delitelj
 	mov CX,5		; maximalan broj cifara
 deli:  xor DX,DX		; DX = 0
 	div BX			; AX = DX:AX/10 , DX = DX:AX mod 10
 	add DL,"0"		; cifra iz DL se prevodi u ascii karakter
 	mov [DI],DL		; i cuva na svoje mesto u memoriji
 	dec DI			; pozicija za sledecu (visu) cifru
 	cmp AX,0		; poredi ceo deo kolicnika sa nulom
 	loopne deli		; ako ima jos cifara i ako je kolicnik <> 0 idi na sledecu cifru
 pisi: 	mov DX,DI		; pozicija pre najvise nenulte cifre
 	inc DX			; pozicija najvise nenulte cifre
 	mov ah,9		; ispis broja u ASCII formatu (string)
 	int 21h
 	ret
 PisiBP ENDP
;,-----------------------------------------------------------------,
;| Ispis obavestenja o gresci pri otvaranju fajla     		   |
;|  * ulaz:							   |
;|    - poruka u memoriji na labeli greskaopen			   |
;|  * izlaz:							   |
;|  * korisceni (izmenjeni) registri:				   |	
;|    - DX,AH                 					   |
;\_________________________________________________________________'
 GrOtv  PROC near
 	mov DX,offset greskaopen	; "greska pri otvaranju fajla"
 	mov AH,9
 	int 21h
 	ret
 GrOtv  endp 	
;,-----------------------------------------------------------------,
;| Ispis obavestenja o gresci pri citanju iz fajla     		   |
;|  * ulaz:							   |
;|    - poruka u memoriji na labeli greskaread			   |
;|  * izlaz:							   |
;|  * korisceni (izmenjeni) registri:				   |	
;|    - DX,AH                 					   |
;\_________________________________________________________________'
 GrCit  PROC near
 	mov DX,offset greskaread	; "greska pri citanju iz fajla"
  	mov AH,9
  	int 21h 	
  	ret
 GrCit	endp  	
   	
 KOD ENDS			; kraj KOD segmenta
 END				; kraj fajla